#!/usr/bin/python
#/usr/bin/env python3

"""Send the contents of a directory as a MIME message."""

import os
import sys
import smtplib
import PyPDF2

# For guessing MIME type based on file name extension
import mimetypes

from argparse import ArgumentParser

from email import encoders
from email.message import Message
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

COMMASPACE = ', '

def set_password(input_file, user_pass, owner_pass):
    """
    Function creates new temporary pdf file with same content,
    assigns given password to pdf and rename it with original file.
    """
    # temporary output file with name same as input file but prepended
    # by "temp_", inside same direcory as input file.
    path, filename = os.path.split(input_file)
    output_file = os.path.join(path, "temp_" + filename)

    output = PyPDF2.PdfFileWriter()

    input_stream = PyPDF2.PdfFileReader(open(input_file, "rb"))

    for i in range(0, input_stream.getNumPages()):
        output.addPage(input_stream.getPage(i))

    outputStream = open(output_file, "wb")

    # Set user and owner password to pdf file
    output.encrypt(user_pass, owner_pass, use_128bit=True)
    output.write(outputStream)
    outputStream.close()

    # Rename temporary output file with original filename, this
    # will automatically delete temporary file
    os.rename(output_file, input_file)

def main():
    parser = ArgumentParser(description="""\
Send the contents of a directory as a MIME message.
Unless the -o option is given, the email is sent by forwarding to your local
SMTP server, which then does the normal delivery process.  Your local machine
must be running an SMTP server.
""")
    parser.add_argument('-d', '--directory',
                        help="""Mail the contents of the specified directory,
                        otherwise use the current directory.  Only the regular
                        files in the directory are sent, and we don't recurse to
                        subdirectories.""")
    parser.add_argument('-H', '--htmlbody',
                        required=False, default='',
                        help="""Mail body in html file. (optional)"""
                        )
    parser.add_argument('-o', '--output',
                        metavar='FILE',
                        help="""Print the composed message to FILE instead of
                        sending the message to the SMTP server.""")
    parser.add_argument('-e', '--encoding', required=False, default='UTF-8',
                        help='Body encoding, default UTF-8')
    parser.add_argument('-p', '--password', required=False, default='',
                        help='Password for encrypting pdf attachment (pdf only)')
    parser.add_argument('-s', '--sender', required=True,
                        help='The value of the From: header (required)')
    parser.add_argument('-t', '--returnpath', required=False,
                        action='append',metavar='Return-Path',
                        default=[], dest='returnpaths',
                        help='The value of the Return-Path: header (optional)')
    parser.add_argument('-r', '--recipient', required=True,
                        action='append', metavar='RECIPIENT',
                        default=[], dest='recipients',
                        help='A To: header value (at least one required)')
    parser.add_argument('-c', '--cc', required=False,
                        action='append', metavar='CC',
                        default=[], dest='cc_recipients',
                        help='A CC: header value (optional)')
    parser.add_argument('-b', '--bcc', required=False,
                        action='append', metavar='BCC',
                        default=[], dest='bcc_recipients',
                        help='A BCC: header value (optional)')
    parser.add_argument('-S', '--subject', required=True,
                        help='The value of the Subject: header (required)')
    args = parser.parse_args()
    directory = args.directory
    if not directory:
        directory = '.'
    # Create the enclosing (outer) message
    args.recipients=','.join(args.recipients).split(',')

    if '' in args.cc_recipients:
	args.cc_recipients.remove('')
    if '' in args.bcc_recipients:
	print('jest pusta')
	args.bcc_recipients.remove('')

    if len(args.cc_recipients)>0:
	args.cc_recipients=','.join(args.cc_recipients).split(',')
    if len(args.bcc_recipients)>0:
	args.bcc_recipients=','.join(args.bcc_recipients).split(',')
#    print(args.recipients) 
#    print(args.cc_recipients)
#    print(args.bcc_recipients)

    outer = MIMEMultipart()
    outer['Subject'] = args.subject
    outer['To'] = COMMASPACE.join(args.recipients)
    outer['From'] = args.sender
    outer['Cc'] = COMMASPACE.join(args.cc_recipients)
    outer['Bcc'] = COMMASPACE.join(args.bcc_recipients)
    # outer['Return-Path'] = COMMASPACE.join(args.returnpaths)
    outer.preamble = 'You will not see this in a MIME-aware mail reader.\n'
    if os.path.isfile(args.htmlbody):
	filename = args.htmlbody.split('/')[-1:][0]
	maintype, subtype = 'text','html'
	encoding = args.encoding
        with open(args.htmlbody) as fp:
            msg = MIMEText(fp.read(), _subtype=subtype, _charset=encoding)
	    msg.add_header('Content-Disposition', 'inline')
            outer.attach(msg)
	# print msg
	
    for filename in os.listdir(directory):
        path = os.path.join(directory, filename)
        if not os.path.isfile(path):
            continue
        #  encrypt with password (pdf only)
        if args.password != '' and filename.lower().endswith('.pdf'):
            set_password(path, args.password, args.password)

        # Guess the content type based on the file's extension.  Encoding
        # will be ignored, although we should check for simple things like
        # gzip'd or compressed files.
        ctype, encoding = mimetypes.guess_type(path)
        if ctype is None or encoding is not None:
            # No guess could be made, or the file is encoded (compressed), so
            # use a generic bag-of-bits type.
            ctype = 'application/octet-stream'
        maintype, subtype = ctype.split('/', 1)
        if maintype == 'text':
            with open(path) as fp:
                # Note: we should handle calculating the charset
                msg = MIMEText(fp.read(), _subtype=subtype)
        elif maintype == 'image':
            with open(path, 'rb') as fp:
                msg = MIMEImage(fp.read(), _subtype=subtype)
        elif maintype == 'audio':
            with open(path, 'rb') as fp:
                msg = MIMEAudio(fp.read(), _subtype=subtype)
        else:
            with open(path, 'rb') as fp:
                msg = MIMEBase(maintype, subtype)
                msg.set_payload(fp.read())
            # Encode the payload using Base64
            encoders.encode_base64(msg)
        # Set the filename parameter
        msg.add_header('Content-Disposition', 'attachment', filename=filename)
        outer.attach(msg)
    # Now send or store the message
    composed = outer.as_string()
    if args.output:
        with open(args.output, 'w') as fp:
            fp.write(composed)
    else:
#        with smtplib.SMTP('localhost') as server:
#            server.sendmail(args.sender, args.recipients + args.cc_recipients + args.bcc_recipients, composed)
        server=smtplib.SMTP('localhost')
#        print(args.recipients + args.cc_recipients + args.bcc_recipients + args.returnpaths)
        server.sendmail(args.sender, args.recipients + args.cc_recipients + args.bcc_recipients, composed)
        server.quit()


if __name__ == '__main__':
    main()
